/*
* Copyright 2010,2013 EUROPEAN UNION
* Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
* the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* http://ec.europa.eu/idabc/eupl
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* Date: 14/06/2013
* Authors: European Commission, Joint Research Centre
* Daniele Francioli, Emanuela Epure
* inspire-registry-info@jrc.ec.europa.eu
*
* This work was supported by the EU  Interoperability Solutions for
* European Public Administrations Programme (http://ec.europa.eu/isa)
* through Action 1.17: Re-usable INSPIRE Reference Platform 
* (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
*/

package eu.europa.ec.re3gistry.re3gistryexport.xsltrasformations;

import eu.europa.ec.re3gistry.re3gistryexport.manager.Configuration;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.ServletContext;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

public class TransformerMgr {

    private static final long serialVersionUID = 1L;
    private static final String KEY_EXPORT_MAINFOLDER_CUSTOM = "export.mainfolder.custom";
    private static final String KEY_EXPORT_MAINFOLDER_NAME = "export.mainfolder.name";
    private static final String KEY_EXPORT_STANDARD_FOLDER_NAME = "export.folder.standard";
    private static final String KEY_EXPORT_CUSTOM_FOLDER_NAME = "export.folder.custom";
    private static final String KEY_EXPORT_FOLDERIDENTIFIER = "export.context.folderidentifier";
    private static String standardFolder;
    private static String customFolder;
    private static String exportMainfolderCustom;
    private static String exportMainfolderName;
    private static String contextFolderIdentifier;
    private static List<File> folderList;
    private static TransformerFactory factory;
    private String folderSeparator = System.getProperty("file.separator");

    public TransformerMgr(ServletContext sc) throws Exception {
        /**
         * Inizializing the transformer
         */
        factory = new net.sf.saxon.TransformerFactoryImpl();

        /**
         * Getting properties
         */
        exportMainfolderCustom = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_CUSTOM, null);
        exportMainfolderName = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_NAME, "export");
        standardFolder = Configuration.getInstance().getProperty(KEY_EXPORT_STANDARD_FOLDER_NAME, "standard");
        customFolder = Configuration.getInstance().getProperty(KEY_EXPORT_CUSTOM_FOLDER_NAME, "custom");
        contextFolderIdentifier = Configuration.getInstance().getProperty(KEY_EXPORT_FOLDERIDENTIFIER, "WEB-INF");
        standardFolder = (standardFolder != null) ? standardFolder.trim() : null;
        customFolder = (customFolder != null) ? customFolder.trim() : null;
        contextFolderIdentifier = (contextFolderIdentifier != null) ? contextFolderIdentifier.trim() : null;
    }

    public void transformRecursively(File rootFolder, String xslFile, String outFileExtension, int folderDepth, ServletContext svc) throws Exception {
        if (factory != null) {
            /**
             * Initializes variables
             */
            List<File> fl = new ArrayList<File>();
            fl.add(rootFolder);

            if (folderDepth < 0) {
                folderList = new ArrayList<File>();
                /**
                 * Scan all levels
                 */
                fl = this.scanAllDirectories(rootFolder);
            } else {
                /**
                 * Specific level scan
                 */
                for (int i = 0; i < folderDepth; i++) {
                    fl = this.scanDirectories(fl);
                }
            }

            /**
             * For each directory, transform the xml files
             */
            for (int i = 0; i < fl.size(); i++) {
                this.transform(fl.get(i), xslFile, outFileExtension, svc);
            }
        } else {
            throw new Exception("An error occurred while starting the transformations: TrasformationMgr has to be inizialized first!");
        }
    }

    public void transform(File folder, String xslFile, String outFileExtension, ServletContext svc) throws Exception {
        /**
         * Get all the file item in this folder
         */
        File[] fileList = folder.listFiles();
        /**
         * For each file item in this folder
         */
        for (File file : fileList) {
            /**
             * Check if the file item is a file
             */
            if (file.isFile()) {
                /**
                 * Get the filename and create the filename of the output file
                 */
                String filename = file.getAbsolutePath();
                String newfilename = filename.replace(".xml", "." + outFileExtension);
                /**
                 * Creating folder for the custom format export
                 */
                newfilename = newfilename.replace(standardFolder, customFolder);
                /**
                 * If the file is an xml file, it has to be converted
                 */
                if (filename.endsWith(".xml")) {
                    /**
                     * Creating directories if needed
                     */
                    String newfolder = newfilename.substring(0, newfilename.lastIndexOf(folderSeparator));
                    if (!new File(newfolder).exists()) {
                        new File(newfolder).mkdirs();
                    }
                    /**
                     * Transform the file and write the new one
                     */
                    Transformer transformer = factory.newTransformer(new StreamSource(xslFile));

                    /**
                     * Passing DB export main folder
                     */
                    String folderDbExport = null;
                    if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
                        /**
                         * If it is setted the custom export mainfolder, it
                         * creates the folders name (from properties)
                         */
                        folderDbExport = exportMainfolderCustom + folderSeparator + exportMainfolderName + folderSeparator + standardFolder;
                    } else {
                        /**
                         * Else get the servlet context path and then creates
                         * the folders name (from properties)
                         */
                        String relativePath = svc.getRealPath(contextFolderIdentifier);
                        folderDbExport = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName + folderSeparator + standardFolder;
                    }
                    transformer.setParameter("base_db_export_path", folderDbExport.replace("\\", "/"));

                    transformer.transform(new StreamSource(filename), new StreamResult(new File(newfilename)));
                }
            }
        }
    }

    private static String readFile(String path) throws IOException {
        FileInputStream stream = new FileInputStream(new File(path));
        try {
            FileChannel fc = stream.getChannel();
            MappedByteBuffer bb = fc.map(FileChannel.MapMode.READ_ONLY, 0, fc.size());
            /**
             * Instead of using default, pass in a decoder.
             */
            return Charset.defaultCharset().decode(bb).toString();
        } finally {
            stream.close();
        }
    }

    private List scanDirectories(List<File> inFolder) throws Exception {
        /**
         * Scanning the directories in order to find subdirectories This
         * function return the list of subdirectories (in all directories passed
         * by parameter)
         */
        folderList = new ArrayList<File>();
        for (File f : inFolder) {
            File[] fl = f.listFiles();
            for (File ff : fl) {
                if (ff.isDirectory()) {
                    folderList.add(ff);
                }
            }
        }
        return folderList;
    }

    private List scanAllDirectories(File inFolder) throws Exception {
        //**Scanning the directories recursively in order to find subdirectories*/
        File[] fl = inFolder.listFiles();
        for (File f : fl) {
            if (f.isDirectory()) {
                folderList.add(f);
                scanAllDirectories(f);
            }
        }
        return folderList;
    }
}